#include <ESP8266WiFi.h>
#include <WiFiUdp.h>
#include <Syslog.h>
#include <Wire.h>  // Only needed for Arduino 1.6.5 and earlier

// Stuff for the temp/humidity sensor
#include "DHT.h"
#define DHTPIN D1     // what digital pin the sensor is connected to
#define DHTTYPE DHT22   // DHT 11

//#define DISPLAY  // Enable if there's an OLED display attached
//#define SERIAL   // To enable serial output
#define WIFI_SSID "WIFI SSID GOES HERE"
#define WIFI_PASS "WIFI PASSWORD GOES HERE"
#define SYSLOG_SERVER "SYSLOG SERVER IP GOES HERE"
#define SYSLOG_PORT 514
#define DEVICE_HOSTNAME "outside" // Inside/outside/garage/shed/etc.
#define APP_NAME "temp-humidity"  // Change this, or keep it. This is what I use so Rsylsog can filter on it.


#ifdef DISPLAY
#include "SSD1306.h" // alias for `#include "SSD1306Wire.h"
// Initialize the OLED display using Wire library
SSD1306  display(0x3c, D3, D5); // These are the pins for the OLED. They should probably be static instead.
#endif

DHT dht(DHTPIN, DHTTYPE); // For the sensor
WiFiUDP udpClient; // A UDP instance to let us send and receive packets over UDP

// Create a new syslog instance with LOG_KERN facility
Syslog syslog(udpClient, SYSLOG_SERVER, SYSLOG_PORT, DEVICE_HOSTNAME, APP_NAME, LOG_KERN);

void setup() {
#ifdef SERIAL
  Serial.begin(115200);
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(WIFI_SSID);
#endif

  WiFi.begin(WIFI_SSID, WIFI_PASS);

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
#ifdef SERIAL
    Serial.print(".");
#endif
  }
#ifdef SERIAL
  Serial.println("");
  Serial.println("WiFi connected");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());
#endif

  // Initialising the UI will init the display too.
#ifdef DISPLAY
  display.init();
  display.flipScreenVertically();
  display.setFont(ArialMT_Plain_10);
  display.setTextAlignment(TEXT_ALIGN_LEFT);
#endif

dht.begin();
}

void loop() {
  float h = dht.readHumidity();;
  float t = dht.readTemperature(); // Read temperature as Celsius (the default)
  float f = dht.readTemperature(true); // Read temperature as Fahrenheit (isFahrenheit = true)
  if (isnan(h) || isnan(t) || isnan(f)) {
#ifdef SERIAL
    Serial.println("Failed to read from DHT sensor!");
#endif
    return;
  }

  // Compute heat index in Fahrenheit (the default)
  float hif = dht.computeHeatIndex(f, h);
  // Compute heat index in Celsius (isFahreheit = false)
  float hic = dht.computeHeatIndex(t, h, false);
#ifdef SERIAL
  Serial.print("Humidity: ");
  Serial.print(h);
  Serial.print(" %\t");
  Serial.print("Temperature: ");
  Serial.print(t);
  Serial.print(" *C ");
  Serial.print(f);
  Serial.print(" *F\t");
  Serial.print("Heat index: ");
  Serial.print(hic);
  Serial.print(" *C ");
  Serial.print(hif);
  Serial.println(" *F");
#endif

#ifdef DISPLAY
  display.clear();
  display.setTextAlignment(TEXT_ALIGN_LEFT);
  display.setFont(ArialMT_Plain_16);
  display.drawString(0, 0, "Temp: " + String(f) + "°");
  display.setFont(ArialMT_Plain_10);
  int i = 6;
  display.drawString(0, 16 + i, "Humidity: " + String(h) + "%");
  display.drawString(0, 28 + i, "Heat index: " + String(hif) + "°");
  //display.drawString(0, 40 + i, "Time:" + String(time));
  // Draw box at top
  display.drawRect(0, 0, DISPLAY_WIDTH, 16);
  display.display(); // Send buffer to display
#endif

  syslog.log(LOG_INFO, String(f) + "," + String(h)); //Where all the syslog magic happens
  delay(1000); // Not sure if this is actually necessary

  // Sleep for 10 minutes before waking up and sending a log message again
  ESP.deepSleep(6e8);
}
